@@ -1,18 +1,20 @@
-# *****************************************************************************
+################################################################################
 #
 # Copyright (c) 2019, the Perspective Authors.
 #
 # This file is part of the Perspective library, distributed under the terms of
 # the Apache License 2.0.  The full license can be found in the LICENSE file.
 #
-import six
 import os
+from datetime import date, datetime
+from io import StringIO
+
 import numpy as np
 import pandas as pd
 import pytz
-from io import StringIO
-from datetime import date, datetime
 from perspective.table import Table
+from pytest import mark
+
 IS_WIN = os.name == 'nt'
 
 
@@ -114,7 +116,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": "string1"}, {"a": 1, "b": "string2"}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         assert view.to_records() == [
             {"__ROW_PATH__": [], "a": 2, "b": 2}, {"__ROW_PATH__": [1], "a": 2, "b": 2}
@@ -124,8 +126,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": "string1"}, {"a": 1, "b": "string2"}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         assert view.to_records() == [
             {"__ROW_PATH__": [], "string1|a": 1, "string1|b": 1, "string2|a": 1, "string2|b": 1},
@@ -136,7 +138,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": "string1"}, {"a": 1, "b": "string2"}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"]
+            split_by=["b"]
         )
         assert view.to_records() == [
             {"string1|a": 1, "string1|b": "string1", "string2|a": None, "string2|b": None},
@@ -215,7 +217,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         assert view.to_dict() == {
             "__ROW_PATH__": [[], [1]],
@@ -227,8 +229,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         assert view.to_dict() == {
             "__ROW_PATH__": [[], [1]],
@@ -240,7 +242,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"]
+            split_by=["b"]
         )
         assert view.to_dict() == {
             "2|a": [1, 1],
@@ -251,7 +253,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
+            group_by=["a"],
             columns=[]
         )
         assert view.to_dict() == {"__ROW_PATH__": [[], [1]]}
@@ -260,8 +262,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             columns=[]
         )
         assert view.to_dict() == {
@@ -272,7 +274,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"],
+            split_by=["b"],
             columns=[]
         )
         assert view.to_dict() == {}
@@ -340,7 +342,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         v = view.to_numpy()
         assert np.array_equal(v["__ROW_PATH__"], [[], [1]])
@@ -351,8 +353,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         v = view.to_numpy()
         assert np.array_equal(v["__ROW_PATH__"], [[], [1]])
@@ -363,7 +365,7 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"]
+            split_by=["b"]
         )
         v = view.to_numpy()
         assert np.array_equal(v["2|a"], np.array([1, 1]))
@@ -375,13 +377,7 @@ class TestToFormat(object):
         tbl = Table(df)
         view = tbl.view()
         df2 = view.to_df()
-        if six.PY2:
-            # order not guaranteed if columns are inferred using aggregate dict keys
-            for col in df2.columns:
-                assert col in ["index", "a", "b"]
-        else:
-            assert np.array_equal(df2.columns, pd.Index(["index", "a", "b"], dtype=object))
-
+        assert np.array_equal(df2.columns, pd.Index(["index", "a", "b"], dtype=object))
         assert np.array_equal(df2["a"].values, df["a"].values)
         assert np.array_equal(df2["b"].values, df["b"].values)
 
@@ -392,12 +388,7 @@ class TestToFormat(object):
         tbl = Table(inp)
         view = tbl.view()
         df2 = view.to_df()
-        if six.PY2:
-            # order not guaranteed if columns are inferred using aggregate dict keys
-            for col in df2.columns:
-                assert col in ["index", "a"]
-        else:
-            assert np.array_equal(df2.columns, pd.Index(["index", "a"], dtype=object))
+        assert np.array_equal(df2.columns, pd.Index(["index", "a"], dtype=object))
         assert np.array_equal(df2["a"].values, df["a"].values)
 
     # start_row/end_row
@@ -414,7 +405,7 @@ class TestToFormat(object):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         records = view.to_records(
             end_row=1000
@@ -429,8 +420,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=1000
@@ -552,7 +543,7 @@ class TestToFormat(object):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         records = view.to_records(
             end_col=1000
@@ -567,7 +558,7 @@ class TestToFormat(object):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         records = view.to_records(
             start_col=2,
@@ -579,7 +570,7 @@ class TestToFormat(object):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         records = view.to_records(
             start_col=20,
@@ -591,20 +582,20 @@ class TestToFormat(object):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
         records = view.to_records(
             start_col=0,
             end_col=0
         )
-        assert records == [{}, {}, {}]
+        assert records == [{'__ROW_PATH__': []}, {'__ROW_PATH__': [1.5]}, {'__ROW_PATH__': [3.5]}]
 
     def test_to_records_two_over_max_col(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_col=1000
@@ -637,8 +628,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=12,
@@ -654,8 +645,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=12,
@@ -668,8 +659,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=12,
@@ -682,8 +673,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=12,
@@ -697,8 +688,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             sort=[["a", "desc"]]
         )
         records = view.to_records(
@@ -712,8 +703,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             sort=[["a", "desc"]]
         )
         records = view.to_records(
@@ -728,8 +719,8 @@ class TestToFormat(object):
         tbl = Table(data)
         view = tbl.view(
             columns=[],
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             sort=[["a", "desc"]]
         )
         records = view.to_records(
@@ -743,23 +734,26 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             sort=[["a", "desc"]]
         )
         records = view.to_records(
             start_col=1,
             end_col=2
         )
-        assert records == [{"__ROW_PATH__": []}, {"__ROW_PATH__": [3]}, {"__ROW_PATH__": [1]}]
-        
+        assert records == [
+            {'2|b': 2, '__ROW_PATH__': []},
+            {'2|b': None, '__ROW_PATH__': [3]},
+            {'2|b': 2, '__ROW_PATH__': [1]}
+        ]
 
     def test_to_records_two_sorted_start_end_col_equiv(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             sort=[["a", "desc"]]
         )
         records = view.to_records(
@@ -778,13 +772,7 @@ class TestToFormat(object):
             end_col=2
         )
         # start_col and end_col access columns at that index - dict key order not guaranteed in python2
-        if six.PY2:
-            # in this test, column c comes before b
-            assert records == [{"c": 3}, {"c": 5}]
-            # assert that in the general dataset, just to see if it holds true
-            assert view.to_records() == [{"a": 1, "c": 3, "b": 2}, {"a": 3, "c": 5, "b": 4}]
-        else:
-            assert records == [{"b": 2}, {"b": 4}]
+        assert records == [{"b": 2}, {"b": 4}]
 
     def test_to_records_start_col_end_col_equiv(self):
         data = [{"a": 1, "b": 2, "c": 3}, {"a": 3, "b": 4, "c": 5}]
@@ -818,8 +806,8 @@ class TestToFormat(object):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
         records = view.to_records(
             end_row=12,
@@ -840,13 +828,7 @@ class TestToFormat(object):
             end_col=1.5
         )
         # start_col and end_col access columns at that index - dict key order not guaranteed in python2
-        if six.PY2:
-            # in this test, column c comes before b
-            assert records == [{"c": 3}, {"c": 5}]
-            # assert that in the general dataset, just to see if it holds true
-            assert view.to_records() == [{"a": 1, "c": 3, "b": 2}, {"a": 3, "c": 5, "b": 4}]
-        else:
-            assert records == [{"b": 2}, {"b": 4}]
+        assert records == [{"b": 2}, {"b": 4}]
 
     def test_to_dict_start_col_end_col(self):
         data = [{"a": 1, "b": 2, "c": 3, "d": 4}, {"a": 3, "b": 4, "c": 5, "d": 6}]
@@ -865,236 +847,129 @@ class TestToFormat(object):
 
     def test_to_csv_symmetric(self):
         csv = "a,b\n1,2\n3,4"
-        if six.PY2:
-            csv = unicode(csv)
         df = pd.read_csv(StringIO(csv))
         tbl = Table(df)
         view = tbl.view()
-        if six.PY2:
-            if IS_WIN:
-                assert view.to_csv() == ",a,b,index\r\n0,1,2,0\r\n1,3,4,1\r\n"
-            else:
-                assert view.to_csv() == ",a,b,index\n0,1,2,0\n1,3,4,1\n"
-        else:
-            if IS_WIN:
-                assert view.to_csv() == ",index,a,b\r\n0,0,1,2\r\n1,1,3,4\r\n"
-            else:
-                assert view.to_csv() == ",index,a,b\n0,0,1,2\n1,1,3,4\n"
+        assert view.to_csv() == '"index","a","b"\n0,1,2\n1,3,4\n'
 
     def test_to_csv_int(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,1,2\r\n1,3,4\r\n"
-        else:
-            assert view.to_csv() == ",a,b\n0,1,2\n1,3,4\n"
+        assert view.to_csv() == '"a","b"\n1,2\n3,4\n'
 
     def test_to_csv_float(self):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,1.5,2.5\r\n1,3.5,4.5\r\n"
-        else:
-            assert view.to_csv() == ",a,b\n0,1.5,2.5\n1,3.5,4.5\n"
+        assert view.to_csv() == '"a","b"\n1.5,2.5\n3.5,4.5\n'
 
     def test_to_csv_date(self):
         today = date.today()
-        dt_str = today.strftime("%Y/%m/%d 00:00:00")
+        dt_str = today.strftime("%Y-%m-%d")
         data = [{"a": today, "b": 2}, {"a": today, "b": 4}]
         tbl = Table(data)
         assert tbl.schema()["a"] == date
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,{},2\r\n1,{},4\r\n".format(dt_str, dt_str)
-        else:
-            assert view.to_csv() == ",a,b\n0,{},2\n1,{},4\n".format(dt_str, dt_str)
-
-    def test_to_csv_date_ignore_custom_format(self):
-        today = date.today()
-        dt_str = today.strftime("%Y")
-        data = [{"a": today, "b": 2}, {"a": today, "b": 4}]
-        tbl = Table(data)
-        assert tbl.schema()["a"] == date
-        view = tbl.view()
-        # date_format only applies to `datetime`
-        if IS_WIN:
-            assert view.to_csv(date_format="%Y") == ",a,b\r\n0,{},2\r\n1,{},4\r\n".format(dt_str, dt_str)
-        else:
-            assert view.to_csv(date_format="%Y") == ",a,b\n0,{},2\n1,{},4\n".format(dt_str, dt_str)
+        assert view.to_csv() == '"a","b"\n{},2\n{},4\n'.format(dt_str, dt_str)
 
     def test_to_csv_datetime(self):
         dt = datetime(2019, 3, 15, 20, 30, 59, 6000)
-        dt_str = dt.strftime("%Y/%m/%d %H:%M:%S")
+        dt_str = dt.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
         data = [{"a": dt, "b": 2}, {"a": dt, "b": 4}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,{},2\r\n1,{},4\r\n".format(dt_str, dt_str)
-        else:
-            assert view.to_csv() == ",a,b\n0,{},2\n1,{},4\n".format(dt_str, dt_str)
-
-    def test_to_csv_datetime_custom_format(self):
-        dt = datetime(2019, 3, 15, 20, 30, 59, 6000)
-        dt_str = dt.strftime("%H:%M:%S")
-        data = [{"a": dt, "b": 2}, {"a": dt, "b": 4}]
-        tbl = Table(data)
-        view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv(date_format="%H:%M:%S") == ",a,b\r\n0,{},2\r\n1,{},4\r\n".format(dt_str, dt_str)
-        else:
-            assert view.to_csv(date_format="%H:%M:%S") == ",a,b\n0,{},2\n1,{},4\n".format(dt_str, dt_str)
+        assert view.to_csv() == '"a","b"\n{},2\n{},4\n'.format(dt_str, dt_str)
 
     def test_to_csv_bool(self):
         data = [{"a": True, "b": False}, {"a": True, "b": False}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,True,False\r\n1,True,False\r\n"
-        else:
-            assert view.to_csv() == ",a,b\n0,True,False\n1,True,False\n"
+        assert view.to_csv() == '"a","b"\ntrue,false\ntrue,false\n'
 
     def test_to_csv_string(self):
         data = [{"a": "string1", "b": "string2"}, {"a": "string3", "b": "string4"}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,string1,string2\r\n1,string3,string4\r\n"
-        else:
-            assert view.to_csv() == ",a,b\n0,string1,string2\n1,string3,string4\n"
+        assert view.to_csv() == '"a","b"\n"string1","string2"\n"string3","string4"\n'
 
     def test_to_csv_none(self):
         data = [{"a": None, "b": None}, {"a": None, "b": None}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv() == ",a,b\r\n0,,\r\n1,,\r\n"
-        else:
-            assert view.to_csv() == ",a,b\n0,,\n1,,\n"
+        assert view.to_csv() == '"a","b"\n,\n,\n'
 
     def test_to_csv_custom_rows(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv(start_row=1) == ",a,b\r\n0,3,4\r\n"
-        else:
-            assert view.to_csv(start_row=1) == ",a,b\n0,3,4\n"
+        assert view.to_csv(start_row=1) == '"a","b"\n3,4\n'
 
     def test_to_csv_custom_cols(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv(start_col=1) == ",b\r\n0,2\r\n1,4\r\n"
-        else:
-            assert view.to_csv(start_col=1) == ",b\n0,2\n1,4\n"
+        assert view.to_csv(start_col=1) == '"b"\n2\n4\n'
 
     def test_to_csv_custom_rows_cols(self):
         data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
         tbl = Table(data)
         view = tbl.view()
-        if IS_WIN:
-            assert view.to_csv(start_row=1, start_col=1) == ",b\r\n0,4\r\n"
-        else:
-            assert view.to_csv(start_row=1, start_col=1) == ",b\n0,4\n"
+        assert view.to_csv(start_row=1, start_col=1) == '"b"\n4\n'
 
     def test_to_csv_one(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"]
+            group_by=["a"]
         )
-        if six.PY2:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__,a,b\r\n0,[],2,4\r\n1,[1],2,4\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__,a,b\n0,[],2,4\n1,[1],2,4\n"
-        else:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__,a,b\r\n0,[],2,4\r\n1,[1],2,4\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__,a,b\n0,[],2,4\n1,[1],2,4\n"
+        assert view.to_csv() == '"a (Group by 1)","a","b"\n,2,4\n1,2,4\n'
 
     def test_to_csv_two(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"]
+            group_by=["a"],
+            split_by=["b"]
         )
-        if six.PY2:
-            if IS_WIN:
-                assert view.to_csv() == ",2|a,2|b,__ROW_PATH__\r\n0,2,4,[]\r\n1,2,4,[1]\r\n"
-            else:
-                assert view.to_csv() == ",2|a,2|b,__ROW_PATH__\n0,2,4,[]\n1,2,4,[1]\n"
-        else:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__,2|a,2|b\r\n0,[],2,4\r\n1,[1],2,4\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__,2|a,2|b\n0,[],2,4\n1,[1],2,4\n"
+        assert view.to_csv() == '"a (Group by 1)","2|a","2|b"\n,2,4\n1,2,4\n'
 
     def test_to_csv_column_only(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"]
+            split_by=["b"]
         )
-        if IS_WIN:
-            assert view.to_csv() == ",2|a,2|b\r\n0,1,2\r\n1,1,2\r\n"
-        else:
-            assert view.to_csv() == ",2|a,2|b\n0,1,2\n1,1,2\n"
+        assert view.to_csv() == '"2|a","2|b"\n1,2\n1,2\n'
 
     def test_to_csv_one_no_columns(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
+            group_by=["a"],
             columns=[]
         )
-        if six.PY2:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__\r\n0,[]\r\n1,[1]\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__\n0,[]\n1,[1]\n"
-        else:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__\r\n0,[]\r\n1,[1]\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__\n0,[]\n1,[1]\n"
+        assert view.to_csv() == '"a (Group by 1)"\n\n1\n'
 
     def test_to_csv_two_no_columns(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            row_pivots=["a"],
-            column_pivots=["b"],
+            group_by=["a"],
+            split_by=["b"],
             columns=[]
         )
-        if six.PY2:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__\r\n0,[]\r\n1,[1]\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__\n0,[]\n1,[1]\n"
-        else:
-            if IS_WIN:
-                assert view.to_csv() == ",__ROW_PATH__\r\n0,[]\r\n1,[1]\r\n"
-            else:
-                assert view.to_csv() == ",__ROW_PATH__\n0,[]\n1,[1]\n"
+        assert view.to_csv() == '"a (Group by 1)"\n\n1\n'
 
     def test_to_csv_column_only_no_columns(self):
         data = [{"a": 1, "b": 2}, {"a": 1, "b": 2}]
         tbl = Table(data)
         view = tbl.view(
-            column_pivots=["b"],
+            split_by=["b"],
             columns=[]
         )
 
-        if IS_WIN:
-            assert view.to_csv() == '""\r\n'
-        else:
-            assert view.to_csv() == '""\n'
+        assert view.to_csv() == ''
 
     # implicit index
 
@@ -1117,10 +992,29 @@ class TestToFormat(object):
             "b": [2.5, 4.5]
         }
 
+    def test_to_format_implicit_id_records(self):
+        data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
+        tbl = Table(data)
+        view = tbl.view()
+        assert view.to_records(id=True) == [
+            {"__ID__": [0], "a": 1.5, "b": 2.5},
+            {"__ID__": [1], "a": 3.5, "b": 4.5}
+        ]
+
+    def test_to_format_implicit_id_dict(self):
+        data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
+        tbl = Table(data)
+        view = tbl.view()
+        assert view.to_dict(id=True) == {
+            "__ID__": [[0], [1]],
+            "a": [1.5, 3.5],
+            "b": [2.5, 4.5]
+        }
+
     def test_to_format_implicit_index_two_dict(self):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
-        view = tbl.view(row_pivots=["a"], column_pivots=["b"])
+        view = tbl.view(group_by=["a"], split_by=["b"])
         assert view.to_dict(index=True) == {
             '2.5|a': [1.5, 1.5, None],
             '2.5|b': [2.5, 2.5, None],
@@ -1130,6 +1024,19 @@ class TestToFormat(object):
             '__ROW_PATH__': [[], [1.5], [3.5]]
         }
 
+    def test_to_format_implicit_index_two_dict(self):
+        data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
+        tbl = Table(data)
+        view = tbl.view(group_by=["a"], split_by=["b"])
+        assert view.to_dict(id=True) == {
+            '2.5|a': [1.5, 1.5, None],
+            '2.5|b': [2.5, 2.5, None],
+            '4.5|a': [3.5, None, 3.5],
+            '4.5|b': [4.5, None, 4.5],
+            '__ID__': [[], [1.5], [3.5]],  # index needs to be the same length as each column
+            '__ROW_PATH__': [[], [1.5], [3.5]]
+        }
+
     def test_to_format_implicit_index_np(self):
         data = [{"a": 1.5, "b": 2.5}, {"a": 3.5, "b": 4.5}]
         tbl = Table(data)
